@@ -11,7 +11,7 @@
 # as Perl itself.
 
 #####################################################################
-require "./libs/xreader.pl";
+if(!($webtools::loaded_functions & 8)) {require "./libs/xreader.pl";}
 $webtools::loaded_functions = $webtools::loaded_functions | 32;
 # $bool = CheckLength($int_var, $min_val, $max_val);
 sub CheckLength
